#pragma once

#include <thread>
#include <memory>
#include <mutex>

#include "clientapi.h"
#include "config.h"
#include "datadragon.h"

class LolAutoAccept {
public:
	using onposchange_func = std::function<void(Position)>;
protected:
	struct Stage {
		Stage();
		virtual ~Stage();

		std::vector<uint32_t> champids; // not every stage has a champ
		bool enabled = false;
		uint32_t currentOffset = 0;
	};

	std::mutex stagesMutex; // protects stagesvector
	std::vector<Stage> stages;
	Position currentPosition = Position::INVALID;

	Config::RootConfig& config;
	DataDragon& dd;
	onposchange_func onPoschange;

	bool shouldrun = false;
	std::thread lolaathread;

	std::shared_ptr<ClientAPI> clientapi;

public:
	enum class State {
		LOBBY = 0,
		BAN = 1,
		PICK = 2,
	};

	LolAutoAccept(Config::RootConfig& config, DataDragon& dd, onposchange_func = {});
	~LolAutoAccept();

	void setChamps(const std::vector<uint32_t>& champs, State s);
	void setEnabled(bool b, State s);

	bool init(); // returns true on success
	void run();
	void stop();

	void reload(); // reload the config, when something was changed

private:
	void stopJoinThread();
	void innerRun();
	void resetAllOffsets();
	void applyConfigToStage(Stage& stage, const Config::StageConfig& stageconf);
	void loadPosition(Position pos);

	uint32_t getChampOfState(State s);
	void nextChampOfState(State s);

	static bool isChampIntentofTeammate(uint32_t champid, const ClientAPI::ChampSelectSession& session);
	static bool isChampBanned(uint32_t champid, const ClientAPI::ChampSelectSession& session);

	using ownactions_t = std::vector<ClientAPI::ChampSelectAction>;
	ownactions_t getOwnActions(int32_t cellid, const std::vector<ClientAPI::ChampSelectAction> actions);
	void prepickPhase(const ownactions_t& ownactions);
	void banPhase(const ownactions_t& ownactions, const ClientAPI::ChampSelectSession& session);
	void pickPhase(const ownactions_t& ownactions);
	void phase(const ownactions_t& ownactions, ClientAPI::ChampSelectActionType type, State s, bool complete, std::function<bool(uint32_t)> filter = {});
	void champSelect();
};