#include <thread>

#include <limits.h>

#include <QApplication>
#include <QFile>
#include <QMessageBox>
#include <QTranslator>

#include <Log.h>

#include "arg.h"
#include "mainwindow.h"
#include "clientaccess.h"
#include "clientapi.h"

static std::string getBaseString(char** argv) {
	std::string base;

	char* appbase = getenv("APPDIR");
	if(appbase) {
		return std::string(appbase) + '/';
	}

	char* cresolved = realpath(argv[0], NULL);
	std::string resolved(cresolved);
	free(cresolved);
	return resolved.substr(0, resolved.rfind('/')+1);
}

int main(int argc, char** argv) {
	Log::init();
	Log::setConsoleLogLevel(Log::Level::INFO);
#if __unix__
	Log::setColoredOutput(true);
#endif

	if(argc == 0) {
		Log::fatal << "arg[0] is not set";
		return 1;
	}

	Args args = parseArgs(argc, argv);
	if(args.debugLog) {
		Log::setConsoleLogLevel(Log::Level::TRACE);
		Log::addLogfile("log.txt", Log::Level::TRACE);
		Log::debug << "debug Log enabled";
	}

	Log::info << "Hello, World!";
	Log::note << "Using Locale: " << QLocale().name().toStdString();

	std::string base = getBaseString(argv);
	Log::info << "appbase: " << base;

	QApplication app(argc, argv);
	QTranslator translator;
	if(translator.load(QLocale().name(), QString::fromStdString(base + "ts"))) {
		app.installTranslator(&translator);
	} else {
		Log::warn << "translation not found";
	}
	MainWindow win;
	QIcon icon(QString::fromStdString(base + "lolautoaccept.png"));
	win.setWindowIcon(icon);

	win.show();
	int ret = app.exec();

	Log::stop();
	return ret;
}